@@ -9,7 +9,8 @@ import json
 class Receipt:
     '''Make a receipt for a function call. Uses string representation of the
     function's arguments.'''
-    root_dir = './receipts'
+    ROOT_DIR = './receipts'
+    TEST = False
     def __init__(self, name='', *a, __dir__=None, **kw):
         if callable(name):
             name = getattr(name, '__qualname__') or getattr(name, '__name__')
@@ -18,8 +19,8 @@ class Receipt:
         self.id = (name or '') + hashlib.md5((
             str(a) + str(sorted(kw.items()))
         ).encode()).hexdigest()
-        self.root_dir = __dir__ or self.root_dir
-        self.fname = os.path.join(self.root_dir, self.id)
+        self.ROOT_DIR = __dir__ or self.ROOT_DIR
+        self.fname = os.path.join(self.ROOT_DIR, self.id)
 
     def __str__(self):
         return '<Receipt exists={} file={}>'.format(
@@ -30,7 +31,7 @@ class Receipt:
         return os.path.isfile(self.fname)
 
     def make(self, **meta):
-        os.makedirs(self.root_dir, exist_ok=True)
+        os.makedirs(self.ROOT_DIR, exist_ok=True)
         with open(self.fname, 'w') as f:
             os.utime(self.fname)
             try:
@@ -58,12 +59,21 @@ class Receipt:
                     return s
 
 
+
+def fallbacks(*xs):
+    for x in xs:
+        if x is not None:
+            return x
+
+
 def use_receipt(func, receipt_dir=None, test=None):
     @functools.wraps(func)
-    def inner(*a, overwrite_=False, test=None, **kw):
-        r = Receipt(func, *a, **kw)
+    def inner(*a, overwrite_=False, test=None, receipt_dir_=None, **kw):
+        test = fallbacks(test, inner.TEST, Receipt.TEST)
+        receipt_dir_ = fallbacks(receipt_dir_, inner.ROOT_DIR, Receipt.ROOT_DIR)
+        r = Receipt(func, *a, __dir__=receipt_dir_, **kw)
         name = r.name
-        if test or inner.TEST:
+        if test:
             print('''
 ------------------------
 -- Test Run --
@@ -113,8 +123,23 @@ Receipt: {}
 Skipping.
 ------------------------
             '''.format(name, r, (r.meta or {}).get('duration_secs')))
-    inner.TEST = use_receipt.TEST if test is None else test
-    inner.RECEIPT_DIR = use_receipt.RECEIPT_DIR if receipt_dir is None else receipt_dir
+    inner.TEST, inner.ROOT_DIR = test, receipt_dir
     return inner
-use_receipt.TEST = False
-use_receipt.RECEIPT_DIR = None
+
+
+# so that setting attributes will set receipt instead
+class _DeprecatedSetAttr:
+    func = None
+    def __init__(self, func):
+        self.func = func
+        functools.update_wrapper(self, func)
+
+    def __call__(self, *a, **kw):
+        return self.func(*a, **kw)
+
+    def __setattr__(self, k, v):
+        print(k, v, k not in self.__dict__)
+        if k not in self.__dict__:
+            setattr(Receipt, k, v)
+            return
+        super().__setattr__(k, v)
